@@ -1,6 +1,7 @@
 # import all the important classes and functions from the bicycleparametes module
 from main import Bicycle
-from plot import plot_eigenvalues
+from plot import plot_eigenvalues, compare_bode_bicycles
+from tables import Table
 
 # the modules that are imported when 'from bicycleparameters import *'
 __all__ = ['main',
@@ -11,8 +12,9 @@ __all__ = ['main',
            'bicycle',
            'com',
            'inertia',
-           'plot']
+           'plot',
+           'tables']
 
 # specify the version for the package
-__version_info__ = (0, 1, 3)
+__version_info__ = (0, 2, 0)
 __version__ = '.'.join(map(str, __version_info__))
