@@ -7,7 +7,8 @@ import os
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.patches import Ellipse, Wedge
-from uncertainties import umath, unumpy
+from uncertainties import unumpy
+from dtk import control
 
 # local module imports
 import bicycle
@@ -40,10 +41,11 @@ class Bicycle(object):
             else:
                 raise ValueError
         except:
-            a = "Are you nuts?! Make a directory called {0} ".format(bicycleName)
-            b = "with basic data for your bicycle in {0}. ".format(pathToData)
-            c = "Then I can actually created a bicycle object."
-            print a + b + c
+            mes = """Are you nuts?! Make a directory called '{0}' with basic data
+for your bicycle in this directory: '{1}'. Then I can actually
+create a bicycle object. You may either need to change to the
+correct directory or reset the pathToData argument.""".format(bicycleName, pathToData)
+            print mes
             return None
 
     def __init__(self, bicycleName, pathToData='.', forceRawCalc=False,
@@ -137,7 +139,14 @@ class Bicycle(object):
             stmt1 = "Looks like you've already got some parameters for %s, "
             stmt2 = "use forceRawCalc to recalculate."
             print (stmt1 + stmt2) % self.bicycleName
-            pass
+            # load the measured.txt file if it exists
+            pathToRawFile = os.path.join(rawDataDir,
+                    self.bicycleName + 'Measured.txt')
+            try:
+                self.parameters['Measured'] = \
+                        io.load_parameter_text_file(pathToRawFile)
+            except IOError:
+                pass
         else:
             print '''There is no data available. Create
             bicycles/{sn}/Parameters/{sn}Benchmark.txt and/or fill
@@ -212,10 +221,14 @@ class Bicycle(object):
 
         """
         photoDir = os.path.join(self.directory, 'Photos')
-        if os.path.isdir(photoDir):
-            os.system('eog ' + os.path.join(photoDir, '*.*'))
-        else:
-            print "There are no photos of your bicycle."
+        try:
+            if os.path.isdir(photoDir):
+                os.system('eog ' + os.path.join(photoDir, '*.*'))
+            else:
+                print "There are no photos of your bicycle."
+        except:
+            raise NotImplementedError("This works only works for linux with " +
+                    "Eye of Gnome installed.")
 
     def steer_assembly_moment_of_inertia(self, handlebar=True, fork=True,
             wheel=True, aboutSteerAxis=False, nominal=False):
@@ -250,10 +263,18 @@ class Bicycle(object):
         the 2 component is perpendicular to the steer axis (pointing to the
         right).
 
+        This function does not currently take into account the flywheel, D, if
+        it is defined, beware.
+
         """
         # load in the Benchmark parameter set
         par = self.parameters['Benchmark']
 
+        if 'mD' in par.keys():
+            print("You have a flywheel defined. Beware that it is ignored in "
+                + "the calculations and the results do not reflect that it is "
+                + "there.")
+
         # there should always be either an H (handlebar/fork) and sometimes
         # there is a G (handlebar) and S (fork) if the fork and handlebar were
         # measured separately
@@ -329,18 +350,9 @@ class Bicycle(object):
             iAssRot = inertia.rotate_inertia_tensor(I, par['lam'])
 
         if aboutSteerAxis:
-            # now find the inertia about the steer axis
-            pointOnAxis1 = np.array([par['w'] + par['c'],
-                                     0.,
-                                     0.])
-            pointOnAxis2 = pointOnAxis1 +\
-                           np.array([-umath.sin(par['lam']),
-                                     0.,
-                                     -umath.cos(par['lam'])])
-            pointsOnLine = np.array([pointOnAxis1, pointOnAxis2]).T
-
             # this is the distance from the assembly com to the steer axis
-            distance = geometry.point_to_line_distance(cAss, pointsOnLine)
+            distance = geometry.distance_to_steer_axis(par['w'], par['c'],
+                    par['lam'], cAss)
             print "handlebar cg distance", distance
 
             # now calculate the inertia about the steer axis of the rotated frame
@@ -472,6 +484,11 @@ class Bicycle(object):
         '''Returns a figure showing the basic bicycle geometry, the centers of
         mass and the moments of inertia.
 
+        Notes
+        -----
+        If the flywheel is defined, it's center of mass corresponds to the
+        front wheel and is not depicted in the plot.
+
         '''
         par = io.remove_uncertainties(self.parameters['Benchmark'])
         parts = get_parts_in_parameters(par)
@@ -500,7 +517,7 @@ class Bicycle(object):
                 Ip, C = inertia.principal_axes(I)
                 if part == 'R':
                     center = np.array([0., par['rR']])
-                elif part == 'F':
+                elif part in 'FD':
                     center = np.array([par['w'], par['rF']])
                 else:
                     center = np.array([par['x' + part], -par['z' + part]])
@@ -599,7 +616,7 @@ class Bicycle(object):
                             color=partColors['R'])
             plt.text(0. + sRad, par['rR'] + sRad, 'R')
             for j, part in enumerate([x for x in parts
-                                      if x != 'R' and x != 'F']):
+                                      if x not in 'RFD']):
                 xcom = par['x' + part]
                 zcom = par['z' + part]
                 ax = com_symbol(ax, (xcom, -zcom), sRad,
@@ -617,44 +634,66 @@ class Bicycle(object):
         # if there is a rider on the bike, make a simple stick figure
         if self.human:
             human = self.human
-            # K2: lower leg
-            plt.plot([human.k[7].pos[0, 0], human.K2.pos[0, 0]],
-                     [-human.k[7].endpos[2, 0], -human.K2.pos[2, 0]], 'k')
-            # K1: upper leg
-            plt.plot([human.K2.pos[0, 0], human.K1.pos[0, 0]],
-                     [-human.K2.pos[2, 0], -human.K1.pos[2, 0]], 'k')
+            mpar = self.parameters['Measured']
+            bpar = self.parameters['Benchmark']
+            # K2: lower leg, tip of foot to knee
+            start = rider.yeadon_vec_to_bicycle_vec(human.K2.end_pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.K2.pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
+            # K1: upper leg, knee to hip
+            start = rider.yeadon_vec_to_bicycle_vec(human.K2.pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.K1.pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
             # torso
-            plt.plot([human.K1.pos[0, 0], human.B1.pos[0, 0]],
-                     [-human.K1.pos[2, 0], -human.B1.pos[2, 0]], 'k')
+            start = rider.yeadon_vec_to_bicycle_vec(human.K1.pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.B1.pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
             # B1: upper arm
-            plt.plot([human.B1.pos[0, 0], human.B2.pos[0, 0]],
-                     [-human.B1.pos[2, 0], -human.B2.pos[2, 0]], 'k')
-            # B2: lower arm
-            plt.plot([human.B2.pos[0, 0], human.b[6].pos[0, 0]],
-                     [-human.B2.pos[2, 0], -human.b[6].endpos[2, 0]], 'k')
+            start = rider.yeadon_vec_to_bicycle_vec(human.B1.pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.B2.pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
+            # B2: lower arm, elbow to tip of fingers
+            start = rider.yeadon_vec_to_bicycle_vec(human.B2.pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.B2.end_pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
             # C: chest/head
-            plt.plot([human.B1.pos[0, 0], human.C.endpos[0, 0]],
-                     [-human.B1.pos[2, 0], -human.C.endpos[2, 0]], 'k')
+            start = rider.yeadon_vec_to_bicycle_vec(human.B1.pos, mpar, bpar)
+            end = rider.yeadon_vec_to_bicycle_vec(human.C.end_pos, mpar, bpar)
+            plt.plot([start[0, 0], end[0, 0]],
+                     [-start[2, 0], -end[2, 0]], 'k')
 
         if show:
             fig.show()
 
         return fig
 
-    def canonical(self):
+    def canonical(self, nominal=False):
         """
-        Returns the canonical velocity and gravity independent matrices for the
-        Whipple bicycle model.
+        Returns the canonical velocity and gravity independent matrices for
+        the Whipple bicycle model linearized about the nominal
+        configuration.
+
+        Parameters
+        ----------
+        nominal : boolean, optional
+            The default is false and uarrays are returned with the
+            calculated uncertainties. If true ndarrays are returned without
+            uncertainties.
 
         Returns
         -------
-        M : ndarray, shape(2,2)
+        M : uarray, shape(2,2)
             Mass matrix.
-        C1 : ndarray, shape(2,2)
+        C1 : uarray, shape(2,2)
             Velocity independent damping matrix.
-        K0 : ndarray, shape(2,2)
+        K0 : uarray, shape(2,2)
             Gravity independent part of the stiffness matrix.
-        K2 : ndarray, shape(2,2)
+        K2 : uarray, shape(2,2)
             Velocity squared independent part of the stiffness matrix.
 
         Notes
@@ -680,15 +719,27 @@ class Bicycle(object):
         v
             Bicylce speed.
 
+        If you have a flywheel defined, body D, it will completely be
+        ignored in these results. These results are strictly for the Whipple
+        bicycle model.
+
         """
 
         par = self.parameters['Benchmark']
 
         M, C1, K0, K2 = bicycle.benchmark_par_to_canonical(par)
 
-        return M, C1, K0, K2
+        if nominal is True:
+            return (unumpy.nominal_values(M),
+                    unumpy.nominal_values(C1),
+                    unumpy.nominal_values(K0),
+                    unumpy.nominal_values(K2))
+        elif nominal is False:
+            return M, C1, K0, K2
+        else:
+            raise ValueError('nominal must be True or False')
 
-    def state_space(self, speed):
+    def state_space(self, speed, nominal=False):
         """
         Returns the A and B matrices for the Whipple model linearized about
         the upright constant velocity configuration.
@@ -698,6 +749,9 @@ class Bicycle(object):
         ----------
         speed : float
             The speed of the bicycle.
+        nominal : boolean, optional
+            The default is false and uarrays are returned with the calculated
+            uncertainties. If true ndarrays are returned without uncertainties.
 
         Returns
         -------
@@ -715,14 +769,18 @@ class Bicycle(object):
 
         where
 
-        The states are [roll rate,
-                        steer rate,
-                        roll angle,
-                        steer angle]
+        The states are [roll angle,
+                        steer angle,
+                        roll rate,
+                        steer rate]
 
         The inputs are [roll torque,
                         steer torque]
 
+        If you have a flywheel defined, body D, it will completely be ignored
+        in these results. These results are strictly for the Whipple bicycle
+        model.
+
         """
 
         M, C1, K0, K2 = self.canonical()
@@ -731,10 +789,16 @@ class Bicycle(object):
 
         A, B = bicycle.ab_matrix(M, C1, K0, K2, speed, g)
 
-        return A, B
+        if nominal is True:
+            return (unumpy.nominal_values(A), unumpy.nominal_values(B))
+        elif nominal is False:
+            return A, B
+        else:
+            raise ValueError('nominal must be True or False')
 
     def eig(self, speeds):
-        '''Returns eigenvalues and eigenvectors of the benchmark bicycle.
+        '''Returns the eigenvalues and eigenvectors of the Whipple bicycle
+        model linearized about the nominal configuration.
 
         Parameters
         ----------
@@ -748,6 +812,12 @@ class Bicycle(object):
         evecs : ndarray, shape (n, 4, 4)
             eigenvectors
 
+        Notes
+        -----
+        If you have a flywheel defined, body D, it will completely be ignored
+        in these results. These results are strictly for the Whipple bicycle
+        model.
+
         '''
         # this allows you to enter a float
         try:
@@ -771,9 +841,9 @@ class Bicycle(object):
         return evals, evecs
 
     def plot_eigenvalues_vs_speed(self, speeds, fig=None, generic=False,
-                                  color='black', show=True, largest=False,
+                                  color='black', show=False, largest=False,
                                   linestyle='-'):
-        '''Returns a plot of the eigenvalues versus speed for the current
+        """Returns a plot of the eigenvalues versus speed for the current
         benchmark parameters.
 
         Parameters
@@ -783,21 +853,27 @@ class Bicycle(object):
         fig : matplotlib figure, optional
             A figure to plot to.
         generic : boolean
-            If true the lines will all be the same color and the modes will not
-            be labeled.
+            If true the lines will all be the same color and the modes will
+            not be labeled.
         color : matplotlib color
             If generic is true this will be the color of the plot lines.
         largest : boolean
             If true, only the largest eigenvalue is plotted.
 
-        '''
+        Notes
+        -----
+        If you have a flywheel defined, body D, it will completely be
+        ignored in these results. These results are strictly for the Whipple
+        bicycle model.
+
+        """
 
         # sort the speeds in case they aren't
         speeds = np.sort(speeds)
 
         # figure properties
         figwidth = 6. # in inches
-        goldenMean = (np.sqrt(5.)-1.) / 2.
+        goldenMean = (np.sqrt(5.0) - 1.0) / 2.0
         figsize = [figwidth, figwidth * goldenMean]
         params = {#'backend': 'ps',
             'axes.labelsize': 8,
@@ -812,7 +888,7 @@ class Bicycle(object):
         if not fig:
             fig = plt.figure(figsize=figsize)
 
-        plt.axes([0.125, 0.2, 0.95-0.125, 0.85-0.2])
+        plt.axes([0.125, 0.2, 0.95 - 0.125, 0.85 - 0.2])
 
         evals, evecs = self.eig(speeds)
 
@@ -886,6 +962,90 @@ class Bicycle(object):
 
         return fig
 
+    def plot_bode(self, speed, u, y, **kwargs):
+        """Returns a Bode plot.
+
+        Parameters
+        ----------
+        speed : float
+            The speed at which to evaluate the system.
+        u : integer
+            An integer between 0 and 1 corresponding to the inputs roll torque
+            and steer torque.
+        y : integer
+            An integer between 0 and 3 corresponding to the inputs roll angle
+            steer angle, roll rate, steer rate.
+        kwargs : keyword pairs
+            Any options that can be passed to dtk.bode.
+
+        Returns
+        -------
+        mag : ndarray, shape(1000,)
+            The magnitude in dB of the frequency reponse.
+        phase : ndarray, shape(1000,)
+            The phase in degress of the frequency response.
+        fig : matplotlib figure
+            The Bode plot.
+
+        """
+
+        A, B = self.state_space(speed, nominal=True)
+
+        C = np.eye(A.shape[0])
+        D = np.zeros_like(B)
+
+        w = np.logspace(0, 2, 1000)
+
+        outputNames = ['Roll Angle', 'Steer Angle', 'Roll Rate', 'Steer Rate']
+        inputNames = ['Roll Torque', 'Steer Torque']
+
+        if 'title' not in kwargs.keys():
+            kwargs['title'] = inputNames[u] + ' to ' + outputNames[y]
+
+        bode = control.bode((A, B[:, u], C[y, :], D[y, u]), w, **kwargs)
+
+        return bode
+
+    def compare_bode_speeds(self, speeds, u, y, fig=None):
+        """Returns a figure with the Bode plots of multiple bicycles.
+
+        Parameters
+        ----------
+        speeds : list
+            A list of speeds at which to evaluate the system.
+        u : integer
+            An integer between 0 and 1 corresponding to the inputs roll torque
+            and steer torque.
+        y : integer
+            An integer between 0 and 3 corresponding to the inputs roll angle,
+            steer angle, roll rate, steer rate.
+
+        Returns
+        -------
+        fig : matplotlib.Figure instance
+            The Bode plot.
+
+        Notes
+        -----
+        The phases are matched around zero degrees at with respect to the first
+        frequency.
+
+        """
+
+        if fig is None:
+            fig = plt.figure()
+
+        for speed in speeds:
+            self.plot_bode(speed, u, y, label=str(speed) + ' m/s', fig=fig)
+
+        # take care of phase misalignment
+        phaseLines = fig.ax2.lines
+        for line in phaseLines:
+            firstValue = line.get_ydata()[0]
+            n = np.ceil(np.floor(abs(firstValue / 180.)) / 2.)
+            line.set_ydata(line.get_ydata() - np.sign(firstValue) * n * 360.)
+        return fig
+
 def get_parts_in_parameters(par):
     '''Returns a list of parts in a parameter dictionary.
 
@@ -898,7 +1058,7 @@ def get_parts_in_parameters(par):
     -------
     parts : list
         Unique list of parts that contain one or more of 'H', 'B', 'F', 'R',
-        'S', 'G'.
+        'S', 'G', 'D'.
 
     '''
     parts = [x[1] for x in par.keys() if x.startswith('m')]
@@ -931,6 +1091,13 @@ def calculate_benchmark_from_measured(mp):
     par['mB'] = mp['mB']
     par['mF'] = mp['mF']
     par['mR'] = mp['mR']
+    try:
+        # we measured the mass of the flywheel plus the mass of the front
+        # wheel, mp['mD'], so to get the actual mass of the flywheel, subtract
+        # the mass of the front wheel
+        par['mD'] = mp['mD'] - mp['mF']
+    except KeyError:
+        pass
     if forkIsSplit:
         par['mS'] = mp['mS']
         par['mG'] = mp['mG']
@@ -957,7 +1124,6 @@ def calculate_benchmark_from_measured(mp):
         par['xH'] = cH[0]
         par['zH'] = cH[2]
 
-
     # local accelation due to gravity
     par['g'] = mp['g']
 
@@ -966,6 +1132,12 @@ def calculate_benchmark_from_measured(mp):
                                             mp['lF'], mp['TcF1'])
     par['IRyy'] = inertia.compound_pendulum_inertia(mp['mR'], mp['g'],
                                             mp['lR'], mp['TcR1'])
+    try:
+        # we measured the inertia of the front wheel with the flywheel inside
+        iFlywheelPlusFwheel = inertia.compound_pendulum_inertia(mp['mD'], mp['g'], mp['lF'], mp['TcD1'])
+        par['IDyy'] = iFlywheelPlusFwheel - par['IFyy']
+    except KeyError:
+        pass
 
     # calculate the y inertias for the frame and fork
     lB = (par['xB']**2 + (par['zB'] + par['rR'])**2)**(0.5)
@@ -1001,6 +1173,10 @@ def calculate_benchmark_from_measured(mp):
     # calculate the wheel x/z inertias
     par['IFxx'] = inertia.tor_inertia(torStiff, mp['TtF1'])
     par['IRxx'] = inertia.tor_inertia(torStiff, mp['TtR1'])
+    try:
+        par['IDxx'] =  inertia.tor_inertia(torStiff, mp['TtD1']) - par['IFxx']
+    except KeyError:
+        pass
 
     pendulumInertias = {}
 
